// AUTOGENERATED FILE - DO NOT MODIFY!
// This file was generated by Djinni from MODULE

#include "java/jni/TtsBackendModule.hpp"  // my header
#include "java/support/jni/djinni_support.hpp"
#include "AudioCallback.hpp"
#include "AbstractTextToSpeechBackend.hpp"

namespace prism::jni {

static void registerModuleNatives(JNIEnv* env, jclass clazz)
{
    djinni::jniRegisterNatives(env, "com/github/ethindp/prism/AudioCallback$CppProxy", AudioCallbackProxyRecords);
}
static const JNINativeMethod moduleMethod[1] = 
{
    {
        const_cast<char*>("initialize"), const_cast<char*>("()V"), reinterpret_cast<void*>(registerModuleNatives)
    }
};
static djinni::JNIMethodLoadAutoRegister moduleLoader("com/github/ethindp/prism/TtsBackendModule$Guard", moduleMethod);

} // namespace prism::jni
