/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_github_ethindp_prism_tests_android_simpleapp_PrismNative */

#ifndef _Included_com_github_ethindp_prism_tests_android_simpleapp_PrismNative
#define _Included_com_github_ethindp_prism_tests_android_simpleapp_PrismNative
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_github_ethindp_prism_tests_android_simpleapp_PrismNative
 * Method:    prism_init
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_com_github_ethindp_prism_tests_android_simpleapp_PrismNative_prism_1init
  (JNIEnv *, jclass);

/*
 * Class:     com_github_ethindp_prism_tests_android_simpleapp_PrismNative
 * Method:    prism_shutdown
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_github_ethindp_prism_tests_android_simpleapp_PrismNative_prism_1shutdown
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_github_ethindp_prism_tests_android_simpleapp_PrismNative
 * Method:    prism_registry_create_best
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_com_github_ethindp_prism_tests_android_simpleapp_PrismNative_prism_1registry_1create_1best
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_github_ethindp_prism_tests_android_simpleapp_PrismNative
 * Method:    prism_backend_free
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_github_ethindp_prism_tests_android_simpleapp_PrismNative_prism_1backend_1free
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_github_ethindp_prism_tests_android_simpleapp_PrismNative
 * Method:    prism_backend_speak
 * Signature: (JLjava/lang/String;Z)I
 */
JNIEXPORT jint JNICALL Java_com_github_ethindp_prism_tests_android_simpleapp_PrismNative_prism_1backend_1speak
  (JNIEnv *, jclass, jlong, jstring, jboolean);

#ifdef __cplusplus
}
#endif
#endif
