// AUTOGENERATED FILE - DO NOT MODIFY!
// This file was generated by Djinni from tts_backend.djinni

#pragma once

#include "java/support/cpp/DataView.hpp"
#include <cstdint>

namespace prism::java {

class AudioCallback {
public:
    virtual ~AudioCallback() = default;

    virtual void on_audio(int64_t userdata, const ::djinni::DataView & samples, int64_t sample_count, int64_t channels, int64_t sample_rate) = 0;
};

} // namespace prism::java
